#include "hoops_license.h"
#include "HDB.h"
#include "HBaseModel.h"
#include "HBaseView.h"
#include "HIOManager.h"
#include "PointCloudAPI.h"
#include <thread>


struct ExitCode {
	enum {
		Success = 0,
		Bad_Command_Line_Arguments,
		Could_Not_Load_File,
	};
};


static ooc::Env load_ooc (HBaseView & view, char const * file_name)
{
	wchar_t w_file_name[4096];
	std::copy_n(file_name, strlen(file_name) + 1, w_file_name);

	HC_KEY destination = view.GetModelKey();

	HInputHandlerOptions options;
	options.m_pHBaseView = &view;
	options.m_bOOCAutoUpdate = false; // Manually control rendering of OOC data for demo code.
	options.m_bOOCAutoCleanup = false; // Manually destroy OOC data.

	ooc::io::IOResult result = ooc::io::FileInputByKey(w_file_name, destination, options);
	if (result != ooc::io::IOResult_Success) {
		return ooc::Env(); // Return an invalid Env.
	}

	HC_KEY ooc_root = HC_Create_Segment_Key_By_Key(destination, "root");
	ooc::Env env;
	ooc::GetEnv(ooc_root, env));
	return env;
}


static void update (HBaseView & view, ooc::Env env)
{
	view.Update(); // Trigger OOC nodes to start populating.

	while (ooc::PollNodesLoadingOrHaveBeenLoaded(env)) {
		std::this_thread::sleep_for(std::chrono::milliseconds(50));
		view.Update(); // Render any newly populated OOC nodes.
	}
}


int main (int argc, char ** args)
{
	if (argc != 2) {
		return ExitCode::Bad_Command_Line_Arguments;
	}

	HC_Define_System_Options("license = `" HOOPS_LICENSE "`");
	HC_Define_System_Options("multi-threading = full"); // Multi-threading needs to be enabled for asynchronous (default) OOC use.

	HDB hdb;
	hdb.Init();
	HBaseModel model;
	model.Init();
	HBaseView view(&model);
	view.Init();

	char const * file_name = args[1];
	ooc::Env env = load_ooc(view, file_name);
	if (env == ooc::Env()) {
		return ExitCode::Could_Not_Load_File;
	}

	update(view, env);

	HC_Define_Alias("?Picture", view.GetDriverPath());
	HC_Pause();

	ooc::Destroy(env); // Clean up OOC data that was loaded.

	return ExitCode::Success;
}





